"""
__init__.py
===========

Author: Bas van Oostveen
Home: https://bitbucket.org/trbs/stripeol/
License: BSD

"""

__docformat__ = "restructuredtext"
__version__ = "1.0"
__license__ = "BSD License"
__author__ = "Bas van Oostveen"


"""
cli.py
======

Commandline interface for stripeol.
"""

__docformat__ = "restructuredtext"

import os
import sys
import logging
from fnmatch import fnmatch
from optparse import OptionParser

from __init__ import __version__
from stripeol import stripeol

ignore_dirs = [
    '.svn', '.git', '.hg', '.bzr', '.cdv', '.pc', 'CVS', 'RCS', '_darcs',
    '_build', 'autom4te.cache',
]
ignore_postfixes = [
    "*~", "*.swp", "*.bak", ".#*", "core.*"
    "*.pyo", "*.pyc", ".class", "*.o", "*.obj", "*.lo", "*.la", "*.a",
    "*.dll", "*.exe", "*.lib", "*.so",
    "*.rej", "*.egg-info", ".DS_Store",
    "*.gz", "*.bz2", "*.xz", "*.rar", "*.zip", "*.tgz", "*.tbz2", "*.tar.gz",
    "*.tar.bz2", "*.arj", "*.txz", "*.tar.xz",
    "*.jpg", "*.bmp", "*.gif", "*.tif", "*.tiff", "*.png",
]

def ignored(filename, options):
    """ Check if the file should be ignored or not """
    for include in options.includes:
        if fnmatch(filename, include):
            return False
    for exclude in options.excludes:
        if fnmatch(filename, exclude):
            return True
    if options.all_types:
        return False
    for postfix in ignore_postfixes:
        if fnmatch(filename, postfix):
            return True
    return False

def stripfile(path, options):
    logging.info("Stripping file '%s'...", path)
    stripeol(path, options)

def scanpath(path, options):
    if not os.path.exists(path):
        logging.warning("File or directory '%s' does not exist...", path)
        return

    if os.path.isfile(path):
        if not ignored(path, options):
            stripfile(path, options)

    if os.path.isdir(path):
        if not options.recursive:
            logging.info("Skipping directory '%s' recursive not specified...", path)
            return

        for root, dirs, filenames in os.walk(path):
            if any(root.endswith(ignore_dir) for ignore_dir in ignore_dirs):
                logging.info("Skipping directory '%s' ignored...", path)
                continue
            for filename in filenames:
                if ignored(filename, options):
                    logging.info("Skipping file '%s' ignored...", path)
                    continue
                stripfile(os.path.join(root, filename), options)

def main():
    """ stripeol cli main entrypoint """
    usage = "Usage: %prog [options] FILES..."
    parser = OptionParser(prog="stripeol", version="%%prog %s" % __version__)
    parser.add_option("-v", "--verbose", dest="verbose", action="store_true",
                      default=False, help="Verbosity")
    parser.add_option("-a", "--all-types", dest="all_types", action="store_true",
                      default=False, help="All files are stripped. Do not use the default ignored directory like VCS and backup files.")
    parser.add_option("-q", "--quiet", dest="quiet", action="store_true",
                      default=False, help="Don't show any output")
    parser.add_option("-n", "--dry-run", dest="dry_run", action="store_true",
                      default=False, help="Don't change any files")
    parser.add_option("-l", "--log_file", dest="log_file",
                      default=None, help="Log output to file")
    parser.add_option("-e", "--no-empty-line", dest="no_empty_line", action="store_true",
                      default=False, help="Don't strip whitespace from empty lines")
    parser.add_option("-r", "--recursive", dest="recursive", action="store_true",
                      default=False, help="Recursively go through directories")
    parser.add_option("-f", "--force", dest="force", action="store_true",
                      default=False, help="Force stripping of file even if it might be binary")
    parser.add_option("-i", "--include", dest="includes", action="append", default=[],
                      help="Include PATTERN, can be specified multiple times")
    parser.add_option("-x", "--exclude", dest="excludes", action="append", default=[],
                      help="Exclude PATTERN, can be specified multiple times")
    parser.add_option
    (options, args) = parser.parse_args()
    if len(args)<1:
        parser.error("Must specify at least one file or directory...")

    level = logging.WARNING
    if options.quiet:
        level = logging.FATAL
    elif options.verbose:
        level = logging.INFO

    format = "%(message)s"
    logging.basicConfig(format=format, filename=options.log_file, level=level)

    if options.dry_run:
        logging.info("DRY RUN")

    for arg in args:
        scanpath(arg, options)

if __name__=="__main__":
    sys.exit(main())



"""
detect.py

tries to detect if file is text file or not
"""

_text_characters = b'\n\r\t\f\b' + b''.join(chr(i) for i in xrange(32, 127))

def istextfile(filename, buffersize=512):
    """ taken from eli.thegreenplace.net """
    block = open(filename).read(buffersize)
    if not block:
        return True
    if b'\x00' in block:
        return False
    nontext = block.translate(None, _text_characters)
    return float(len(nontext)) / len(block) <= 0.30



#!/usr/bin/env python

from stripeol import __version__, __author__

from setuptools import setup

long_description = open("README").read()

setup(
    name="stripeol",
    license="BSD License",
    version=__version__,
    description="Simple tool to strip whitespace from files",
    long_description=long_description,
    maintainer=__author__,
    author=__author__,
    author_email="trbs @ trbs",
    url="https://bitbucker.org/trbs/stripeol/",
    keywords="strip eol cleanup white space files",
    classifiers=[
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Text Processing",
        "Topic :: Utilities",
        "Environment :: Console",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "Operating System :: Unix",


    ],
    packages=['stripeol'],
    package_dir={'stripeol': 'stripeol'},
    entry_points={'console_scripts': ['stripeol = stripeol.cli:main']},
)



"""
stripeol.py
"""

import os
import sys
import logging

from tempfile import NamedTemporaryFile
try:
    from tempfile import SpooledTemporaryFile
    def TemporaryFile(**kwargs):
        return SpooledTemporaryFile(max_size=4*1024*1024, **kwargs)
except ImportError:
    from tempfile import TemporaryFile

from detect import istextfile

def stripeol(filename, options=None):
    """ stripeol """
    if options is None:
        # default options
        class options:
            force = False
            dry_run = False
            no_empty_line = False

    if os.path.getsize(filename)==0:
        logging.info("Skipping %s empty file...", filename)
        return
    # detect binary file
    if not options.force and not istextfile(filename):
        logging.info("Skipping %s binary file...", filename)
        return

    changed = False
    prefix = os.path.basename(filename)
    dirname = os.path.dirname(os.path.realpath(filename))
    with NamedTemporaryFile(prefix=os.path.basename(filename), dir=dirname, delete=False) as ftmp:
        with open(filename) as rsrc:
            for line in rsrc:
                line = line.rstrip("\n")
                if options.no_empty_line and line.strip()=="":
                    ftmp.write(line+"\n")
                    continue
                strip_line = line.rstrip()
                if not changed and line != strip_line:
                    changed = True
                ftmp.write(strip_line+"\n")
    if options.dry_run:
        os.unlink(ftmp.name)
    else:
        os.rename(ftmp.name, filename)
    if changed:
        logging.warning("Stripped whitespace from: %s", filename)
